#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace Xenoflight
{
    class SpriteManager : DrawableGameComponent
    {
        List<SpaceObject> _objects;
        GameState _gameState;
        SpriteBatch _spriteBatch;
        Dictionary<string, Texture2D> _textureCache;
        Dictionary<Physics2DDotNet.Body, SpaceObject> _physicsLookup;

        public SpriteManager(GameState gs)
            : base(gs._game)
        {
            _gameState = gs;
            _objects = new List<SpaceObject>();
            _textureCache = new Dictionary<string, Texture2D>();
            _physicsLookup = new Dictionary<Physics2DDotNet.Body, SpaceObject>();
            _spriteBatch = new SpriteBatch(_gameState._graphics.GraphicsDevice);
        }

        public void Add(SpaceObject so, string sprite)
        {
            System.IO.StreamReader sr = new System.IO.StreamReader(Paths.Sprites + "\\" + sprite + ".sprite");

            string type = sr.ReadLine();

            if (type == "animated")
                loadAnimated(so, sr);
            else if (type=="static")
            {
                string file = sr.ReadLine();
                so.Texture = loadTexture(Paths.Sprites + "\\" + file);
            }

            sr.Close();

            // Initialize physics on the object
            Physics2DDotNet.Body body = new Physics2DDotNet.Body(new Physics2DDotNet.PhysicsState(), so.CollisionShape, so.Mass, _gameState._physicsCoefficients.Duplicate(), new Physics2DDotNet.Lifespan());
			so.Body = body;
            body.Collided += new EventHandler<Physics2DDotNet.CollisionEventArgs>(Collision);
			so.Body.State.Position = new Physics2DDotNet.Math2D.ALVector2D(0, 5, 5);
			so.State.Velocity = new Physics2DDotNet.Math2D.ALVector2D(0, 0, 0);

            _physicsLookup.Add(body, so);
            _gameState._physicsEngine.AddBody(body);
            _objects.Add(so);
        }

        void Collision(object sender, Physics2DDotNet.CollisionEventArgs e)
        {
            SpaceObject so = _physicsLookup[(Physics2DDotNet.Body)sender];
			foreach (Physics2DDotNet.IContactInfo ci in e.Contacts)
			{
				if (ci.Body1 == so.Body && _physicsLookup.ContainsKey(ci.Body2))
					so.Collided(_physicsLookup[ci.Body2]);
				else if (_physicsLookup.ContainsKey(ci.Body1))
					so.Collided(_physicsLookup[ci.Body1]);
			}
			Log.log("xeno.physics.collisions", "Objects collided!");
        }

        void loadAnimated(SpaceObject so, System.IO.StreamReader sr)
        {
            AnimatedSprite ani = (AnimatedSprite)so;
            Dictionary<int, Texture2D> frames = new Dictionary<int,Texture2D>();

            string basepath = "/", baseext = "";
            while (!sr.EndOfStream)
            {
                string line = sr.ReadLine();
                if (line == "" || line.Substring(0, 1) == "#") continue;
                string[] parts = line.Split(' ');

                switch (parts[0])
                {
                    case "fps":
                        ani.FPS = int.Parse(parts[1]);
                        break;
                    case "base":
                        basepath = parts[1];
                        break;
                    case "ext":
                        baseext = parts[1];
                        break;
                    default:
                        frames[int.Parse(parts[0])] = loadTexture(Paths.Sprites + basepath + parts[1] + baseext);
                        break;
                }
            }
            Texture2D[] ret = new Texture2D[frames.Count];
            int i=0;
            foreach (KeyValuePair<int, Texture2D> kvp in frames)
                ret[i++] = kvp.Value;
            ani.SetFrames(ret);
        }

        public void Remove(SpaceObject so)
        {
			so.Body.Lifetime = new Physics2DDotNet.Lifespan(0);
            _objects.Remove(so);
        }

        Texture2D loadTexture(string fullPath)
        {
            if (_textureCache.ContainsKey(fullPath))
                return _textureCache[fullPath];
            else
            {
                _textureCache.Add(fullPath, Texture2D.FromFile(_gameState._graphics.GraphicsDevice, fullPath));
                return _textureCache[fullPath];
            }
        }

        public override void Draw(GameTime gameTime)
        {
            float drawX = _gameState._drawLocation.X;
            float drawY = _gameState._drawLocation.Y;
            int tWidth = _gameState._tileWidth;
            int tHeight = _gameState._tileHeight;
            // The location and size of the screen with relation to the 0,0 of the map, in pixels.
            System.Drawing.RectangleF screen = new System.Drawing.RectangleF(drawX, drawY, drawX + _gameState._graphics.GraphicsDevice.Viewport.Width, drawY + _gameState._graphics.GraphicsDevice.Viewport.Height);

            _spriteBatch.Begin(SpriteBlendMode.AlphaBlend);
            foreach (SpaceObject so in _objects)
            {
                System.Drawing.RectangleF obj = new System.Drawing.RectangleF (so.Location.X*tWidth-(so.Texture.Width>>1), so.Location.Y*tHeight-(so.Texture.Height>>1), so.Texture.Width, so.Texture.Height);
                if (screen.IntersectsWith(obj))
                    _spriteBatch.Draw(so.Texture, new Vector2(obj.Left - screen.X, obj.Top - screen.Y), so.Tint);
            }
            _spriteBatch.End();

            base.Draw(gameTime);
        }

        public override void Update(GameTime gameTime)
        {
			_gameState._physicsEngine.Update((float)gameTime.ElapsedGameTime.TotalSeconds * _gameState._timeScale);
            foreach (SpaceObject so in _objects)
                so.Update(gameTime);
            base.Update(gameTime);
        }
    }
}
