#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace Xenoflight
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Xenoflight : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;
        GameState gs = new GameState();

        public Xenoflight()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            // Generate initial game state
            gs = new GameState();
            gs._game = this;
            gs._graphics = graphics;

            // Initialize logging
            Dictionary<string, string> facilities = new Dictionary<string,string>();
            facilities.Add("xeno", "xeno");
            facilities.Add("xeno.init", "xeno.init");
            facilities.Add("xeno.engine", "xeno.engine");
            facilities.Add("xeno.physics", "xeno.physics");
			facilities.Add("xeno.physics.collisions", "xeno.physics.collisions");
			facilities.Add("xeno.physics.engine", "xeno.physics.engine");
			facilities.Add("xeno.physics.ship", "xeno.physics.ship");
			facilities.Add("xeno.player", "xeno.player");
			facilities.Add("xeno.player.input", "xeno.player.input");
            Log log = new Log(gs, facilities);
			Components.Add(log);

            Log.log("xeno.init", "Logging subsystem initialized");

            // Initialize the physics engine
            Log.log("xeno.init", "Initializing physics subsystem");
            gs._physicsEngine = new Physics2DDotNet.PhysicsEngine();
#if DEBUG
			Log.log("xeno.init", "	Initializing physics debugging");
			Components.Add(new PhysicsDebugger(gs, gs._physicsEngine, "xeno.physics.engine"));
#endif
            Log.log("xeno.init", "	Creating broad phase collision detector");
            gs._physicsEngine.BroadPhase = new Physics2DDotNet.Detectors.SweepAndPruneDetector();
            Log.log("xeno.init", "	Creating solver");
            Physics2DDotNet.Solvers.SequentialImpulsesSolver sis = new Physics2DDotNet.Solvers.SequentialImpulsesSolver();
            {
                sis.Iterations = 12;
                sis.SplitImpulse = true;
                sis.BiasFactor = 0.7f;
                sis.AllowedPenetration = 0.1f;
            }
            gs._physicsEngine.Solver = sis;
            Log.log("xeno.init", "	Initializing coefficients");
            gs._physicsCoefficients = new Physics2DDotNet.Coefficients(0.5f, 0.4f, 0.4f);

            // Add the input service
			Log.log("xeno.init", "Initializing input subsystem");
            Services.AddService(typeof(InputServices.InputServiceProvider), new InputServices.KeyboardInputService());

            // Create the sprite manager
			Log.log("xeno.init", "Initializing sprite manager");
            gs._spriteManager = new SpriteManager(gs);

            // Create the player's ship and add it to the sprite manager
			Log.log("xeno.init", "	Creating ship");
            Ship ship = new Entities.Ships.Gradius(gs);
            gs._spriteManager.Add(ship, "ship");
#if DEBUG
			Components.Add(new BodyDebugger(gs, ship.Body, "xeno.physics.ship"));
#endif
			ship.Body.State.Position = new Physics2DDotNet.Math2D.ALVector2D(0, 3f, 6);
			ship.Body.State.Velocity = new Physics2DDotNet.Math2D.ALVector2D(0, 3f, 0.1f);

            // Add the player interface component
			Log.log("xeno.init", "	Creating input translator");
			Components.Add(new Player(gs, ship));

            // Create an asteroid and add it to the sprite manager
			Log.log("xeno.init", "	Creating junk");
            SpaceObject asteroid = new Entities.Litter.Asteroid(gs);
            gs._spriteManager.Add(asteroid, "asteroid");
			asteroid.Body.State.Position = new Physics2DDotNet.Math2D.ALVector2D(0, 20, 6);
			asteroid.Body.State.Velocity = new Physics2DDotNet.Math2D.ALVector2D(0, -5f, 0f);

			// Create the FPS display component and add it
			Log.log("xeno.init", "Initializing FPS displayer");
			FPSCounter fps = new FPSCounter(gs);

			// Load the default map and add it to the components
			Log.log("xeno.init", "Loading default map");
			Map map = Map.FromFile(gs, "default.map");
			if (map == null)
				throw new Exception();
			gs._map = map;


			Components.Add(map);
			Components.Add(fps);
			Components.Add(gs._spriteManager);

			Log.log("xeno.init", "Done");

            base.Initialize();
        }
    }
}
